"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _react = _interopRequireWildcard(require("react"));
var _propTypes = _interopRequireDefault(require("prop-types"));
var _classnames = _interopRequireDefault(require("classnames"));
var _Popup = _interopRequireDefault(require("./Popup"));
var _Popover = _interopRequireDefault(require("../../styles/cspace-layout/Popover.css"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function _getRequireWildcardCache(e) { if ("function" != typeof WeakMap) return null; var r = new WeakMap(), t = new WeakMap(); return (_getRequireWildcardCache = function (e) { return e ? t : r; })(e); }
function _interopRequireWildcard(e, r) { if (!r && e && e.__esModule) return e; if (null === e || "object" != typeof e && "function" != typeof e) return { default: e }; var t = _getRequireWildcardCache(r); if (t && t.has(e)) return t.get(e); var n = { __proto__: null }, a = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var u in e) if ("default" !== u && Object.prototype.hasOwnProperty.call(e, u)) { var i = a ? Object.getOwnPropertyDescriptor(e, u) : null; i && (i.get || i.set) ? Object.defineProperty(n, u, i) : n[u] = e[u]; } return n.default = e, t && t.set(e, n), n; }
/* global window */

const focus = component => {
  if (component) {
    component.focus();
  }
};
const propTypes = {
  children: _propTypes.default.node,
  header: _propTypes.default.node,
  align: _propTypes.default.string,
  openDelay: _propTypes.default.number,
  renderContent: _propTypes.default.func,
  onBeforeOpen: _propTypes.default.func,
  onBeforeClose: _propTypes.default.func
};
const defaultProps = {
  children: undefined,
  header: undefined,
  align: 'left',
  openDelay: 250,
  renderContent: undefined,
  onBeforeOpen: undefined,
  onBeforeClose: undefined
};
class Popover extends _react.Component {
  constructor() {
    super();
    this.handleHeaderClick = this.handleHeaderClick.bind(this);
    this.handleHeaderKeyDown = this.handleHeaderKeyDown.bind(this);
    this.handleHeaderMouseEnter = this.handleHeaderMouseEnter.bind(this);
    this.handleHeaderMouseLeave = this.handleHeaderMouseLeave.bind(this);
    this.handlePopupBlur = this.handlePopupBlur.bind(this);
    this.handlePopupKeyDown = this.handlePopupKeyDown.bind(this);
    this.handlePopupMouseLeave = this.handlePopupMouseLeave.bind(this);
    this.state = {
      open: false
    };
  }
  handleHeaderClick(event) {
    event.preventDefault();
    this.open();
  }
  handleHeaderKeyDown(event) {
    if (event.key === 'ArrowDown') {
      event.preventDefault();
      this.open();
    }
  }
  handleHeaderMouseEnter() {
    const {
      openDelay
    } = this.props;
    const {
      open
    } = this.state;
    if (!open) {
      this.openTimer = setTimeout(() => {
        this.open();
      }, openDelay);
    }
  }
  handleHeaderMouseLeave() {
    this.cancelOpen();
  }
  handlePopupBlur() {
    this.close();
  }
  handlePopupKeyDown(event) {
    if (event.key === 'Escape') {
      this.close();
    }
  }
  handlePopupMouseLeave() {
    this.close();
  }
  cancelOpen() {
    if (this.openTimer) {
      window.clearTimeout(this.openTimer);
      this.openTimer = null;
    }
  }
  close() {
    this.cancelOpen();
    const {
      open
    } = this.state;
    if (open) {
      const {
        onBeforeClose
      } = this.props;
      if (onBeforeClose) {
        onBeforeClose();
      }
      this.setState({
        open: false
      });
    }
  }
  open() {
    this.cancelOpen();
    const {
      open
    } = this.state;
    if (!open) {
      const {
        onBeforeOpen
      } = this.props;
      if (onBeforeOpen) {
        onBeforeOpen();
      }
      this.setState({
        open: true
      });
    }
  }
  renderHeader() {
    const {
      header
    } = this.props;
    return /*#__PURE__*/_react.default.createElement("button", {
      type: "button",
      onClick: this.handleHeaderClick,
      onKeyDown: this.handleHeaderKeyDown,
      onMouseEnter: this.handleHeaderMouseEnter,
      onMouseLeave: this.handleHeaderMouseLeave
    }, header);
  }
  renderPopup() {
    const {
      children,
      renderContent
    } = this.props;
    const {
      open
    } = this.state;
    if (open) {
      const content = renderContent ? renderContent() : children;
      if (content) {
        return /*#__PURE__*/_react.default.createElement(_Popup.default, {
          ref: focus,
          tabIndex: "0",
          onBlur: this.handlePopupBlur,
          onKeyDown: this.handlePopupKeyDown,
          onMouseLeave: this.handlePopupMouseLeave
        }, content);
      }
    }
    return null;
  }
  render() {
    const {
      align
    } = this.props;
    const {
      open
    } = this.state;
    const classes = (0, _classnames.default)(_Popover.default.common, _Popover.default[align], {
      [_Popover.default.open]: open
    });
    return /*#__PURE__*/_react.default.createElement("div", {
      className: classes
    }, this.renderHeader(), this.renderPopup());
  }
}
exports.default = Popover;
Popover.propTypes = propTypes;
Popover.defaultProps = defaultProps;